<?php

namespace Imoje\Payment;

/**
 * Class Util
 *
 * @package Imoje\Payment
 */
class Util
{
	// region notification codes
	const NC_OK = 0;
	const NC_INVALID_SIGNATURE = 1;
	const NC_SERVICE_ID_NOT_MATCH = 2;
	const NC_ORDER_NOT_FOUND = 3;
	const NC_INVALID_SIGNATURE_HEADERS = 4;
	const NC_EMPTY_NOTIFICATION = 5;
	const NC_NOTIFICATION_IS_NOT_JSON = 6;
	const NC_INVALID_JSON_STRUCTURE = 7;
	const NC_INVALID_ORDER_STATUS = 8;
	const NC_AMOUNT_NOT_MATCH = 9;
	const NC_UNHANDLED_STATUS = 10;
	const NC_ORDER_STATUS_NOT_CHANGED = 11;
	const NC_CART_NOT_FOUND = 12;
	const NC_ORDER_STATUS_IS_NOT_SETTLED_ORDER_ARRANGEMENT_AFTER_IPN = 13;
	const NC_ORDER_EXISTS_ORDER_ARRANGEMENT_AFTER_IPN = 14;
	const NC_REQUEST_IS_NOT_POST = 15;
	const NC_MISSING_TWISTO_RESPONSE_IN_POST = 16;
	const NC_MISSING_ORDER_ID_IN_POST = 17;
	const NC_CURL_IS_NOT_INSTALLED = 18;
	const NC_MISSING_SIGNATURE_IN_POST = 19;
	const NC_UNKNOWN = 100;
	// endregion

	// region notification status
	const NS_OK = 'ok';
	const NS_ERROR = 'error';
	// endregion

	const METHOD_REQUEST_POST = 'POST';
	const METHOD_REQUEST_GET = 'GET';
	const ENVIRONMENT_PRODUCTION = 'production';
	const ENVIRONMENT_SANDBOX = 'sandbox';

	private static $cdnUrl = 'https://data.imoje.pl';

	/**
	 * @var array
	 */
	private static $supportedCurrencies = [
		'pln' => 'PLN',
		'eur' => 'EUR',
		'czk' => 'CZK',
		'gbp' => 'GBP',
		'usd' => 'USD',
		'uah' => 'UAH',
		'hrk' => 'HRK',
		'huf' => 'HUF',
		'sek' => 'SEK',
		'ron' => 'RON',
		'chf' => 'CHF',
		'bgn' => 'BGN',
	];

	/**
	 * @var array
	 */
	private static $paymentMethods = [
		'blik'     => 'blik',
		'paylater' => 'imoje_paylater',
		'pbl'      => 'pbl',
		'card'     => 'card',
		'ing'      => 'ing',
	];

	/**
	 * @var array
	 */
	private static $paymentMethodCodeList = [
		'blik'     => 'blik',
		'paylater' => 'imoje_twisto',
	];

	/**
	 * @var array
	 */
	private static $paymentMethodCodeLogoExt = [
		'alior'          => 'alior.svg',
		'bnpparibas'     => 'bnpparibas.png',
		'bos'            => 'bos.png',
		'bs'             => 'bs.png',
		'bspb'           => 'bspb.png',
		'bzwbk'          => 'bzwbk.png',
		'citi'           => 'citi.png',
		'creditagricole' => 'creditagricole.svg',
		'envelo'         => 'envelo.png',
		'getin'          => 'getin.svg',
		'ideabank'       => 'ideabank.png',
		'ing'            => 'ing.png',
		'inteligo'       => 'inteligo.png',
		'ipko'           => 'ipko.png',
		'millennium'     => 'millennium.svg',
		'mtransfer'      => 'mtransfer.png',
		'nest'           => 'nest.svg',
		'noble'          => 'noble.png',
		'pbs'            => 'pbs.png',
		'pekao24'        => 'pekao24.svg',
		'plusbank'       => 'plusbank.png',
		'pocztowy'       => 'pocztowy.svg',
		'tmobile'        => 'tmobile.svg',
	];

	/**
	 * @var array
	 */
	private static $hashMethods = [
		'sha224' => 'sha224',
		'sha256' => 'sha256',
		'sha384' => 'sha384',
		'sha512' => 'sha512',
	];

	/**
	 * @var array
	 */
	private static $transactionStatuses = [
		'new'        => 'new',
		'authorized' => 'authorized',
		'pending'    => 'pending',
		'submitted_for_settlement'
		             => 'submitted_for_settlement',
		'rejected'   => 'rejected',
		'settled'    => 'settled',
		'error'      => 'error',
		'cancelled'  => 'cancelled',
	];

	/**
	 * Functions that return true when passed currency is on supported currencies list.
	 *
	 * @param string $currencyCode ISO4217
	 *
	 * @return bool
	 */
	public static function canUseForCurrency($currencyCode, $currencyList)
	{
		return in_array( strtolower($currencyCode), $currencyList);
	}

	/**
	 * @param string $hashMethod
	 *
	 * @return string
	 */
	public static function getHashMethod($hashMethod)
	{

		if(isset(self::$hashMethods[$hashMethod])) {
			return self::$hashMethods[$hashMethod];
		}

		return '';
	}

	/**
	 * @param array  $order
	 * @param string $submitValue
	 * @param string $url
	 *
	 * @return string
	 */
	public static function createOrderForm($order, $url = '', $method = '', $submitValue = '')
	{
		if(!$submitValue) {
			$submitValue = 'Continue';
		}

		if(!$url) {
			$url = '';
		}

		if(!$method) {
			$method = 'POST';
		}

		$form = '<form method="' . $method . '" action="' . $url . '">';

		if(is_array($order)) {
			foreach($order as $key => $value) {
				$form .= '<input type="hidden" value="' . htmlentities($value) . '" name="' . $key . '" id="imoje_' . $key . '">';
			}
		}

		$form .= '<button type="submit" id="submit-payment-form">' . $submitValue . '</button>';
		$form .= '</form>';

		return $form;
	}

	/**
	 * @return array
	 */
	public static function getSupportedCurrencies()
	{
		return self::$supportedCurrencies;
	}

	/**
	 * @param string $paymentMethod
	 *
	 * @return string
	 */
	public static function getPaymentMethod($paymentMethod)
	{

		if(isset(self::$paymentMethods[$paymentMethod])) {
			return self::$paymentMethods[$paymentMethod];
		}

		return '';
	}

	/**
	 * @param string $paymentMethodCode
	 *
	 * @return string
	 */
	public static function getPaymentMethodCodeLogo($paymentMethodCode)
	{

		if(isset(self::$paymentMethodCodeLogoExt[$paymentMethodCode])) {
			return self::$cdnUrl . '/img/pay/' . self::$paymentMethodCodeLogoExt[$paymentMethodCode];
		}

		return '';
	}

	/**
	 * @param string $paymentMethodCode
	 *
	 * @return string
	 */
	public static function getPaymentMethodCode($paymentMethodCode)
	{
		if(isset(self::$paymentMethodCodeList[$paymentMethodCode])) {
			return self::$paymentMethodCodeList[$paymentMethodCode];
		}

		return '';
	}

	/**
	 * @param string $variable
	 *
	 * @return bool
	 */
	public static function isJson($variable)
	{
		json_decode($variable);

		return (json_last_error() === JSON_ERROR_NONE);
	}

	/**
	 * @return array
	 */
	public static function getTransactionStatuses()
	{
		return self::$transactionStatuses;
	}

	/**
	 * @param float $amount
	 *
	 * @return int
	 */
	public static function convertAmountToFractional($amount)
	{
		return (int) self::multiplyValues(round($amount, 2), 100, 0);
	}

	/**
	 * @param number $firstValue
	 * @param number $secondValue
	 * @param number $precision
	 *
	 * @return float
	 */
	public static function multiplyValues($firstValue, $secondValue, $precision)
	{
		return round($firstValue * $secondValue, $precision);
	}

	/**
	 * @param int $amount
	 *
	 * @return float
	 */
	public static function convertAmountToMain($amount)
	{
		return round($amount / 100, 2);
	}

	/**
	 * @param string $hashMethod
	 * @param string $data
	 * @param string $serviceKey
	 *
	 * @return string
	 */
	public static function hashSignature($hashMethod, $data, $serviceKey)
	{
		return hash($hashMethod, $data . $serviceKey);
	}

}
